//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

#include <tigcclib.h>
#include <extgraph.h>

//----------------------------------------------------------------------------------------------------

#define MAZE_W (6)
#define MAZE_H (6)

#define RANDOM_COUNT (256)

//----------------------------------------------------------------------------------------------------

enum {MOV_U,MOV_R,MOV_D,MOV_L};

//----------------------------------------------------------------------------------------------------

void MazeInitialize(void);
void MazeMove(int way, int rank);
void MazeRandom(void);

void DrawScreen(void);

//----------------------------------------------------------------------------------------------------

char*Plane0;

int maze_data[MAZE_H][MAZE_W];
int maze_data2[MAZE_H][MAZE_W];

int CurrentWay=MOV_U;
int CurrentPos=0;
int moves_count=0;

//----------------------------------------------------------------------------------------------------

unsigned short TileData[]={
0xFFF0,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,
0x8000,0x8000,0x8000,0x8000,0x0000,0x0000,0x0000,0x0000,};


unsigned short Arrows_rawData[48] =
{
    0x0600, 0x0F00, 0x1F80, 0x3FC0, 0x7FE0, 0xFFF0, 0x0600, 0x0600, 
    0x0600, 0x0600, 0x0600, 0x0600, 0x0200, 0x0300, 0x0380, 0x03C0, 
    0x03E0, 0xFFF0, 0xFFF0, 0x03E0, 0x03C0, 0x0380, 0x0300, 0x0200, 
    0x0600, 0x0600, 0x0600, 0x0600, 0x0600, 0x0600, 0xFFF0, 0x7FE0, 
    0x3FC0, 0x1F80, 0x0F00, 0x0600, 0x0400, 0x0C00, 0x1C00, 0x3C00, 
    0x7C00, 0xFFF0, 0xFFF0, 0x7C00, 0x3C00, 0x1C00, 0x0C00, 0x0400, 
} ;

//----------------------------------------------------------------------------------------------------

void MazeInitialize(void)
{
	
	int x,y;
	int counter=1;
	
	for (y=0;y<MAZE_H;y++)
	for (x=0;x<MAZE_W;x++)
	maze_data[y][x]=counter++;
	
	moves_count=0;
	
	// 2 = sizeof (int)
	memcpy(maze_data2,maze_data,2*MAZE_H*MAZE_W);
	
}

//----------------------------------------------------------------------------------------------------

void MazeMove(int way, int rank)
{
	int a;
	
	if (way==MOV_L)
	{
		int temp=maze_data[rank][0];
		for (a=0;a<MAZE_W-1;a++)
		maze_data[rank][a]=maze_data[rank][a+1];
		maze_data[rank][MAZE_W-1]=temp;
	}
	
	else if (way==MOV_R)
	{
		int temp=maze_data[rank][MAZE_W-1];
		for (a=MAZE_W-1;a>0;a--)
		maze_data[rank][a]=maze_data[rank][a-1];
		maze_data[rank][0]=temp;
	}
	
	else if (way==MOV_U)
	{
		int temp=maze_data[0][rank];
		for (a=0;a<MAZE_H-1;a++)
		maze_data[a][rank]=maze_data[a+1][rank];
		maze_data[MAZE_H-1][rank]=temp;
	}
	
	else if (way==MOV_D)
	{
		int temp=maze_data[MAZE_H-1][rank];
		for (a=MAZE_H-1;a>0;a--)
		maze_data[a][rank]=maze_data[a-1][rank];
		maze_data[0][rank]=temp;
	}
	
	moves_count++;
}

//----------------------------------------------------------------------------------------------------

void MazeRandom(void)
{
	int a;
	
	for (a=0;a<RANDOM_COUNT;a++)
	{
		int way=random(4);
		if (way==MOV_U || way==MOV_D)
		MazeMove(way,random(MAZE_W));
		else
		MazeMove(way,random(MAZE_H));
	}
	
	moves_count=0;
}
//----------------------------------------------------------------------------------------------------

void DrawScreen(void)
{
	int x,y;
	char buffer[10];
	
	PortSet(Plane0,239,127);
	//GrayClearScreen2B();
	ClrScr();
	
	for (y=0;y<MAZE_H;y++)
	for (x=0;x<MAZE_W;x++)
	{
		int xx=12*(x+1);
		int yy=12*(y+1);
		
		sprintf(buffer,"%d",maze_data[y][x]);
		
		Sprite16_OR(xx,yy,12,TileData,Plane0);
		DrawStr(xx+2,yy+4,buffer,A_NORMAL);
	}
	
	// CurrentWay
	
	unsigned short *src=Arrows_rawData+12*CurrentWay;
	
	if (CurrentWay==MOV_U)
	Sprite16_OR(12*(CurrentPos+1),84,12,src,Plane0);
	else if (CurrentWay==MOV_D)
	Sprite16_OR(12*(CurrentPos+1),0,12,src,Plane0);
	else if (CurrentWay==MOV_L)
	Sprite16_OR(84,12*(CurrentPos+1),12,src,Plane0);
	else if (CurrentWay==MOV_R)
	Sprite16_OR(0,12*(CurrentPos+1),12,src,Plane0);
	
	PortRestore();
	FastCopyScreen(Plane0,LCD_MEM);
	
	
	
}
//----------------------------------------------------------------------------------------------------


// Main Function
void _main(void)
{
	
	Plane0=malloc(LCD_SIZE);
	
	clrscr();
	FontSetSys(0);
	printf("SwapMaze 1.0 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	
	randomize();
	FontSetSys(0);
	
	MazeInitialize();
	DrawScreen();
	DrawStr(100,0,"Ready?",A_NORMAL);
	ngetchx();
	
	MazeRandom();
	
	for (;;)
	{
		DrawScreen();
		
		char buffer[50];
		sprintf(buffer,"Moves : %d",moves_count);
		DrawStr(100,0,buffer,A_NORMAL);
		
		// 2 = sizeof (int)
		if (!memcmp(maze_data,maze_data2,2*MAZE_H*MAZE_W))
		DrawStr(100,10,"You Win!",A_NORMAL);
		
		
		int key=ngetchx();
		
		// KEY_LEFT
		if (key==KEY_LEFT)
		{
			if (CurrentWay==MOV_U || CurrentWay==MOV_D)
			{
				if (CurrentPos)
				CurrentPos--;
			}
			else
			MazeMove(MOV_L,CurrentPos);
		}
		
		// KEY_RIGHT
		else if (key==KEY_RIGHT)
		{
			if (CurrentWay==MOV_U || CurrentWay==MOV_D)
			{
				if (CurrentPos<MAZE_W-1)
				CurrentPos++;
			}
			else
			MazeMove(MOV_R,CurrentPos);
		}
		
		// KEY_UP
		else if (key==KEY_UP)
		{
			if (CurrentWay==MOV_L || CurrentWay==MOV_R)
			{
				if (CurrentPos)
				CurrentPos--;
			}
			else
			MazeMove(MOV_U,CurrentPos);
		}
		
		// KEY_DOWN
		else if (key==KEY_DOWN)
		{
			if (CurrentWay==MOV_L || CurrentWay==MOV_R)
			{
				if (CurrentPos<MAZE_H-1)
				CurrentPos++;
			}
			else
			MazeMove(MOV_D,CurrentPos);
		}
		
		// KEY_SHIFT+...
		else if (key==KEY_SHIFT+KEY_UP)
		CurrentWay=MOV_D;
		else if (key==KEY_SHIFT+KEY_DOWN)
		CurrentWay=MOV_U;
		else if (key==KEY_SHIFT+KEY_LEFT)
		CurrentWay=MOV_R;
		else if (key==KEY_SHIFT+KEY_RIGHT)
		CurrentWay=MOV_L;
		
		else if (key==KEY_APPS)
		off();
		
		else if (key==KEY_F1)
		{
			MazeInitialize();
			MazeRandom();
		}
		
		else if (key==KEY_ESC)
		break;
	}
		
	if (Plane0)
	free (Plane0);
	
	
	
}
//----------------------------------------------------------------------------------------------------
